# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Functionalities of this script were tested for FS6.0.0 hippocampal subfield
# output (v21.txt files).
#
# The script that will (1) generate a histogram plot (in .png format)
# for each hippocampal subfield, (2) output a text file with numerical
# summaries, (3) give warnings in case of missing values or negative values,
# and (4) identify outliers and (5) detect 'rank violations'.
# 
# Original versions by Derrek Hibar (derrek.hibar@ini.usc.edu), Laura van Velzen
# (l.vanvelzen@ggzingeest.nl), Lianne Schmaal (lianschmaal@gmail.com).
# Combined with outlier detection and rank violation detection by
# Philipp G. Sämann (saemann@psych.mpg.de).
# 12/2016
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

# Read in data file
# Set path if needed:
# setwd("O:/AGNMR/saemann/SUBFIELD_scripts_part_IR_II/Entire_Remake_Web/QCpackage_updated_FS6.0_or_FS7.1/FS6.0.0")
dat=read.csv("hippo_subfields.csv")
n_variables <- dim(dat)[2]

# Define list of auto-QC subjects
autolist=NULL

# Check how many subjects to use a good number of bins for hist plots
hbins=NULL;
if(nrow(dat)>200){
	hbins=100;
}

# Get column names
cnames=colnames(dat);

cat('\nCheck for missing values, NA and X/x entries producing histograms.\n\n')

# Create a file to store summary statistics for each structure
write("Structure\tNumberExcluded\tTotalGoodSubjs\tMean\tStandDev", file="SummaryStats_hippo_subfields.txt");

# Loop through each structure including the three global values and produce plots

for(x in 2:length(cnames)){
	
	# Do some checking of the data to make sure it looks ok
	badsegs=0;
	ind=which(dat[,x]=="x")
	ind2=which(dat[,x]=="X")
	index=c(ind,ind2);
	if(length(index) > 0){
		stop("There were values marked with x or X detected in your hippo_subfields.csv file. All poorly segmented \
		or otherwise missing values should be marked with the letters NA in the hippo_subfields.csv file.\n\n");
	}

	# Check to make sure there are not any missing values
	miss=which(dat[,x]=="");
	if(length(miss)>0){
		stop("There were missing values detected in your hippo_subfields.csv file. Open your hippo_subfields.csv file \
		in Excel and locate any blank cells. Missing values should be marked with NA. \n\n");
	}

	# Check to make sure none of the values are negative
	negs=which(dat[,x]<0);
	if(length(negs)>0){
		stop("Some of your volume values are negative. This does not makes sense. Open your hippo_subfields.csv file \
		in Excel and set negative volume values and poorly segmented values to NA in the file.\n\n");
	}

	# Find which subjects are marked as NA for a given structure in the loop
	nas=which(is.na(dat[,x]));
	if(length(nas)>0){
		interm=dat[-nas,x];
		badsegs=badsegs+length(nas)
		cat(paste("You marked ", as.character(badsegs), " subjects as poorly segmented (volumes) in the ", cnames[x], "\n", sep=''));
	} else {
		interm=dat[,x];
		# cat(paste("None of the subjects in the ", cnames[x], " were marked as poorly segmented (volumes)\n", sep=''));
	}
	
	# Make sure data is in numeric format
	interm=as.numeric(as.vector(interm));

	# Get the summary statistic values
	mu=mean(interm);
	sdev=sd(interm);
	n.used=length(interm);
	stats=c(cnames[x], badsegs, n.used, mu, sdev);
	write.table(t(as.matrix(stats)), file="SummaryStats_hippo_subfields.txt", append=T, quote=F, col.names=F,row.names=F,sep="\t");
	png(paste(cnames[x],"_hist_volumes.png",sep=""))
	hist(interm, nclass=hbins, main=cnames[x], xlab="Average Volume");
	dev.off()
}

# Output an R data file to store the plot data
save.image(file="ENIGMA_hippo_subfields_plots.Rdata");
cat('Check passed.\nCases with NA values will be ignored.\n\n')
dat = dat[complete.cases(dat),]

cat('\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n')
cat('%   1. Outlier analysis     %\n')
cat('%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n\n')

# Check for duplicated SubjIDs
if(anyDuplicated(dat[,c("SubjID")]) != 0) { stop('You have duplicate SubjIDs in your *.csv file.\nMake sure there are no repeat SubjIDs.') }

# Store the lower and upper limits
lower=rep(NA,ncol(dat)-1)
upper=rep(NA,ncol(dat)-1)

# Loop through the different structures
for(x in 2:n_variables){
	lower[x-1] = mean(dat[,x]) - 2.698*sd(dat[,x])
	upper[x-1] = mean(dat[,x]) + 2.698*sd(dat[,x])
}

cat('The following volumes lie outside the +/- 2 SDs marks of the (raw value) distribution:\n')

# Loop through each subject report which structures are outliers
for(i in 1:nrow(dat)){
        flag <- 0
	lowind <- which(dat[i,-1] < lower)
	upind  <- which(dat[i,-1] > upper)
	flag <- length(upind) + length(lowind)
	rating <- ''
	if(length(lowind) > 0){
	cat(paste(dat[i,1], ': LOW ', names(dat[i,-1])[lowind], collapse = '\n'), '\n' )
        }

	if(length(upind) > 0){
	  cat(paste(dat[i,1], ': HIGH ', names(dat[i,-1])[upind], collapse = '\n'), '\n')
	}
	if (flag>0){
	autolist <- c(autolist,dat[i,1])
	cat(paste('\n'))
	}
}


cat('\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n')
cat('%    2. Ranking analysis    %\n')
cat('%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n\n')

CA1_check <- 0

n <- dim(dat)[1]

for(i in 1:n){ 

	   flag <- 0  
	   tmp <- dat[i,2:13]
	   tmp2 <- rank(tmp)
 
	   rating <- dat[i,1]

	   # check if CA1 is not largest subregion
	   if ((tmp2[3]!=12) && (CA1_check==1)){
	   flag <- flag + 1
	   rating <- paste(rating,' CA1 not largest;')
	   }

	   # check if hippocampal tail below rank 3
	   if (tmp2[1]<10){
           flag <- flag + 1
           rating <- paste(rating,' hippocampal tail below rank 3;')
           }

	   # check if the subiculum is ranked no. 4
	   if (tmp2[2]!=9){
	   flag <- flag + 1
           rating <- paste(rating,' & subiculum not rank 4')
           }

	   if (flag>0){
	   cat(paste(rating,'\n'))
	   autolist <- c(autolist,dat[i,1])
	   }
     }


cat('\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n')
cat(  '%  Resulting minimum list of cases fom steps 1 & 2 for visual QC  %:\n')
cat(  '%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n')

if (length(autolist)>0){

# check for duplicates

d = duplicated(autolist)

for (i in 1:length(autolist)){
    if (d[i]==FALSE){
    #cat(paste('Perform QC on ', dat[autolist[i],1],'\n'))
    cat(paste('Perform QC on ', autolist[i],'\n'))
    }
    }
}
